import { object, string, boolean } from '@edtr-io/plugin';
import { EditorInlineSettings, EditorInput, EditorCheckbox } from '@edtr-io/editor-ui';
import { styled } from '@edtr-io/ui';
import { createElement, useState, Fragment } from 'react';
import { mergeDeepRight } from 'ramda';
import { PrismAsyncLight } from 'react-syntax-highlighter';
import style from 'react-syntax-highlighter/dist/cjs/styles/prism/coy';

function HighlightRenderer(props) {
  var _useHighlightConfig = useHighlightConfig(props.config),
    i18n = _useHighlightConfig.i18n;
  return createElement(PrismAsyncLight, {
    language: props.language,
    showLineNumbers: props.showLineNumbers,
    style: style,
    customStyle: {
      overflow: 'auto'
    }
  }, props.code || i18n.code.label);
}

function useHighlightConfig(config) {
  var _config$i18n = config.i18n,
    i18n = _config$i18n === void 0 ? {} : _config$i18n,
    _config$Renderer = config.Renderer,
    Renderer = _config$Renderer === void 0 ? HighlightRenderer : _config$Renderer;
  return {
    i18n: mergeDeepRight({
      code: {
        label: 'Click here and enter your source code…',
        placeholder: 'Enter your source code here'
      },
      language: {
        label: 'Language',
        placeholder: 'Enter language'
      },
      showLineNumbers: {
        label: 'Show line numbers'
      }
    }, i18n),
    Renderer: Renderer
  };
}

var Textarea = /*#__PURE__*/styled.textarea({
  height: '250px',
  width: '100%',
  margin: 'auto',
  padding: '10px',
  resize: 'none',
  fontFamily: 'Menlo, Monaco, "Courier New", monospace',
  border: 'none',
  outline: 'none',
  boxShadow: '0 1px 1px 0 rgba(0,0,0,0.50)',
  '&::-webkit-input-placeholder': {
    color: 'rgba(0,0,0,0.5)'
  }
});
var CheckboxContainer = /*#__PURE__*/styled.div({
  "float": 'right'
});
function HighlightEditor(props) {
  var config = props.config,
    state = props.state,
    focused = props.focused,
    editable = props.editable;
  var _useHighlightConfig = useHighlightConfig(config),
    i18n = _useHighlightConfig.i18n,
    Renderer = _useHighlightConfig.Renderer;
  var edit = focused && editable;
  var _React$useState = useState(edit),
    throttledEdit = _React$useState[0],
    setEditThrottled = _React$useState[1];
  if (edit != throttledEdit) {
    if (!edit) {
      setTimeout(function () {
        setEditThrottled(false);
      }, 500);
    } else {
      setEditThrottled(true);
    }
  }
  return throttledEdit || edit ? createElement(Fragment, null, createElement(Textarea, {
    value: state.code.value,
    name: "text",
    placeholder: i18n.code.placeholder,
    onChange: function onChange(e) {
      state.code.set(e.target.value);
    }
  }, state.code.value), createElement(EditorInlineSettings, null, createElement(EditorInput, {
    list: "available-languages",
    label: i18n.language.label,
    value: state.language.value,
    onChange: function onChange(e) {
      state.language.set(e.target.value);
    },
    placeholder: i18n.language.placeholder
  }), createElement("datalist", {
    id: "available-languages"
  }, ['c', 'javascript', 'jsx', 'markup', 'java', 'python'].map(function (language) {
    return createElement("option", {
      key: language,
      value: language
    });
  })), createElement(CheckboxContainer, null, createElement(EditorCheckbox, {
    label: i18n.showLineNumbers.label,
    onChange: function onChange() {
      state.showLineNumbers.set(!state.showLineNumbers.value);
    },
    checked: state.showLineNumbers.value
  })))) : createElement(Renderer, {
    config: config,
    language: state.language.value,
    showLineNumbers: state.showLineNumbers.value,
    code: state.code.value
  });
}

/**
 * @param config - {@link HighlightConfig | Plugin configuration}
 * @public
 */
function createHighlightPlugin(config) {
  if (config === void 0) {
    config = {};
  }
  return {
    Component: HighlightEditor,
    config: config,
    state: object({
      code: string(''),
      language: string('text'),
      showLineNumbers: boolean(false)
    })
  };
}

export { createHighlightPlugin };
//# sourceMappingURL=plugin-highlight.esm.js.map
